/* This is an implementation of shift scheduling problem
 * for demonstration CGI by S. Moriguchi.
 *
 *  c.f. G. Koole and E. van der Sluis (2003): Optimal shift scheduling
 *  with a global service level constraint, IIE Transactions, Vol. 35,
 *  pp. 1049-1055.
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "odicon.h"

static double factorial(int n) {
    static double save[200];

    assert( 0 <= n );
    assert( n < sizeof(save)/sizeof(save[0]) );
    if (save[n] == 0) {
        int i;
        double r = 1;
        for (i=2; i<=n; i++) r *= i;
        save[n] = r;
    }
    return save[n];
}

#define I 13
#define DELTA 0.000005

/* determined in demo_shift_schedule() */
static double mu = 6, c = 11;
static double lambda[I] = {25, 25, 23, 24, 5, 1, 1, 1, 1, 1, 1, 1, 1};

double gi(int s, int i) {
    double LAMBDA=0, g=0, rho=0, beta=0, alpha=0, rho_n=0;
    int j;
    
    rho = lambda[i-1] / mu;
    rho_n = lambda[i-1] / mu / s;
    for(j=0; j<I; j++){
        LAMBDA=LAMBDA+lambda[j];
    }
    
    /*	printf("%d fac=%f \n",i, factorial(0));*/
    
    beta = (pow(rho, s)/factorial(s)/(1-rho/s));
    /*	printf("beta = %f, \n",beta);*/
    
    for(j=0; j<s; j++){
        alpha = alpha + pow(rho,j) / factorial(j);
    }
    /*	printf("alpha = %f, \n",alpha);*/
    
    if (beta < DELTA) {
        g = lambda[i-1]/LAMBDA;
    } else if (alpha < DELTA) {
        g = lambda[i-1]/LAMBDA * (1- exp(-s * mu*(1-rho_n)*c));
    } else {
        g = lambda[i-1]/LAMBDA * (1-beta/(alpha + beta) * exp(-s * mu*(1-rho_n)*c));
    }

    if (g < DELTA) g=0;
    /*	printf("g_%d = %f \n",i,g);
	printf("--------------------\n");*/
    return g;
}


/* f1(x) = \sum g_i */
static double f1(int dim, int x[]) {
    double r = 0;
    int j;
  
    if (x[0] <= 0) return PLUS_INFINITY;
    for (j=1; j<dim; j++) {
        if (x[j] <= 0 || x[j]<=x[j-1]) return PLUS_INFINITY;
    }
    assert(dim == 4);  /* check dimension */
    r +=  gi(x[0],1);
    r +=  gi(x[0],2);
    r +=  gi(x[1],3);
    r +=  gi(x[1],4);
    r +=  gi(x[1],5);
    r +=  gi(x[2]-x[0],6);
    r +=  gi(x[2]-x[0],7);
    r +=  gi(x[2]-x[1],8);
    r +=  gi(x[3]-x[1],9);
    r +=  gi(x[3]-x[1],10);
    r +=  gi(x[3]-x[2],11);
    r +=  gi(x[3]-x[2],12);
    r +=  gi(x[3]-x[2],13);

/*printf("r= %f \n", r);*/
  
    return -r;
}


void demo_shift_schedule(const int dim) {
    int x[4] = {2, 3, 4, 5};  /* initial solution */
    double ans;
    int i;

    assert( dim == 4 );
    /* for (i=0; i<=10; i++) printf("%d! = %f\n", i, factorial(i)); */
    /* printf("gi = %f\n", gi(5,5)); */

    fvector_scan(1, &c);
    fvector_scan(1, &mu);
    fvector_scan(I, lambda);
    ivector_scan(dim, x);

    ans = lconv_minimize(dim, f1, x, 3);
    printf("ans = %g\n", -ans);
    for(i=0; i<4; i++) {
        printf("x[%d] = %d\n", i, x[i]);
    }
}
