#!/usr/bin/perl -w

use CGI qw/:standard *table *Tr *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e) = @_;
    "<img src=\"$e.png\" alt=\"$e\" align=\"middle\">";
}

$NATURAL = '<img src="natural.png">';  # without align

$dim = param('dim');
unless ($dim) {
    $dim = 13;
    param('dim', $dim);
}

print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> "Shift Scheduling Problem: L$NATURAL-convex minimization"),
    a({href=>"./index.en.html"},'DCP HOME'), ln,
    h1("Shift Scheduling Problem: L$NATURAL-convex minimization"), ln,
    '<BLOCKQUOTE>', ln;


sub set_param {
    my ($index, $min, $max) = @_;
    my ($e) = (param($index));
    if ( defined($e) && $e =~ m/([-+]?\d+)/
	 && $min <= int($1) && int($1) <= $max ) {
	$e = int($1);
    } else {
	$e = $min + int(rand($max-$min+1));
    }
    param($index, $e);
}

sub init_param {
    my ($i, $j);

    set_param('criterion', 1, 20);
    set_param('mu', 1, 99);
    for ($i=0; $i<$dim; $i++) {
	set_param("rate:$i", 1, 99);
    }
    for ($i=0; $i<4; $i++) {
	set_param("x:$i", 1, 99);
    }
}

sub print_html_table {
    init_param();
    print <<'EOF';
<p>
We consider the shift scheduling problem
in call centers [1].
Suppose
a call center is operational during I=13 intervals.
During K=4 of these intervals
employees can start working, and each employee works for
M=5 consecutive intervals.
Shift k starts at the I<sub>k</sub>-th interval
and thus finishes at the beginning of interval I<sub>k</sub> +M. We
assume that
  I<sub>1</sub> = 1, I<sub>2</sub> = 3, I<sub>3</sub> = 7, I<sub>4</sub> = 9.
<p><img src="shift/shiftex.png">         
<p>
For each interval i (i=1, . . . , I) there is a function g<sub>i</sub> representing
the service level as a function of the
number of employees working in that interval.
Let x<sub>k</sub> be the number of employees of shift k for k = 1, . . . ,K.
Then the number of employees h<sub>i</sub>(x) working at interval i is given by
<p><img src="shift/hix.png">
<p>
     The value of g<sub>i</sub> at this point is the attained service level in
interval i. The overall service level is defined by
<p><img src="shift/Sxgi.png">
     <p>
Now consider the following problem, which maximizes
the service level for a given number of employees:
<p>     <img src="shift/maxSx.png">     
<p>     
  We assume that call arrivals are Poisson.
The service criterion is the fraction of customers that has to wait
longer than c seconds, e.g., c=11 seconds, before getting an operator,
which should be below 5% in general.
(We assume that no customers leave the system before getting
an operator.)
<p>
     Assuming that a statistical equilibrium is attained in
each time interval, we use the formula for the stationary waiting
time in an M|M|n queue as the service level.
Let
     <img src="shift/def-Lambda.png" align="middle"> <p>
     then the expected service level
of an arbitrary customer under schedule x is equal to
<p>
          <img src="shift/SxPW.png">
<p>     
where      <img src="shift/nihi.png"> 
 and
     <img src="shift/def-W.png"> 
are the waiting time
in an M|M|n queue with arrival rate  <img src="lambda.png"> 
     and service rate
 <img src="shift/mu.png"> .
     Thus we take
<p>      <img src="shift/def-gi.png"> <p>
This function is indeed monotone increasing and concave for each i.
     Therefore,  <img src="shift/minusS.png"> is multimodular. 
We can minimize <img src="shift/minusS.png"> by ODICON because
     multimodular functions and L-convex functions
     are equivalent objects
that can be related through a simple coordinate transformation.

EOF
#    print p, 'n: number of items', ln;
     print start_form, hidden('dim'), ln;
#
#    @index = (1..15);
#    print p, 'n = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
#    print submit('Change');

    print p, image('shift/c'), ': Service criterion (We measure the fraction of
customers that has to wait longer than ' , image('shift/c'), ' seconds before
getting an operator)', br, ln;

    print p, image('shift/c'), ' = ', textfield(-name=>'criterion', -size=>4, -default=>param('criterion')), ln;

    print p, image('shift/mu'),    ': Service rate ',br, ln;
    print p, image('shift/mu'), ' = ', textfield(-name=>'mu', -size=>4, -default=>param('mu')), br, ln;


print image('shift/rate'),    ': Arrival rates', 
    ', 0 &lt; ', image('shift/rate'), '(', image('shift/j'), " = 1, 2, ... ,$dim)",br, ln;

    print start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'}, [ image('shift/j'), 1 .. $dim ])]);
    foreach $e ('rate') {
	my @td = (image("shift/$e"));
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }

    foreach $e ('x') {
	my @td = ($e);
	for ($j=0; $j<4; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }

  
    print end_table, ln;

    print p, 'You can input parameters ',  image('shift/c'),
    ' in the ranges 0 &lt; c &lt; 20',
    ' and ',  image('shift/mu'), ' , ',  image('shift/rate'),
    ' in the ranges 0 &lt; ',  image('shift/mu'), ' , ', image('shift/rate'),
   '&lt; 100. ', image('shift/mu'), ' and ', image('shift/rate'),
    ' should satisfy the condition ', 
  image('shift/rate'), '&lt; ', image('shift/mu'), '.',
#    ', the unsatisfactory condition turns red.',
  ln;
#    print p, 'You can also input an initial solution x.', ln;
  
    print p, '(Note: "Reset" button resets your inputs to default values.', ln;
    print br, 'An inappropriate input value will be replaced with a random number.)', ln;
    print p, submit('Minimize'), reset('Reset'), ln;
    print end_form, ln;

    print br, ln;


#    print p, "We minimize the following cost function f(x):\n";
#    print image('objfunc'), ',', br, ln;
#    print 'where ', image('x'),       ': Spare amount of item ', image('j'),
#    ', 0 &le; ', image('x'), '(', image('j'), ' = 1, 2, ... ,n)',br, ln;
    print <<EOF;

<p>
<!--
The first term is the backorder penalty which is computed as
the cost per backorder times
the steady-state expected number of maximum backorders among the items.
The second term is the cost of spares purchases.

<p>
This function is L$NATURAL-convex [*]. -->
This web application minimizes <img src="shift/minusS.png"> using
<a href="http://www.misojiro.t.u-tokyo.ac.jp/~tutimura/odicon/index.en.html">ODICON</a>
<!--with IFF by Satoru Iwata-->.

<hr width="70%">

<p> [1] G. Koole and E. van der Sluis (2003):
  Optimal shift scheduling with a global service level
constraint,
IIE Transactions,
  Vol. 35, pp. 1049-1055.

<hr width="70%">
EOF
}

sub call_executable {
    print '<pre>', ln;
    open(OUT, "|./odicon DS 4") || die('Can not call ./odicon');
    #open(OUT, '|cat') || die('Can not call ./odicon');
    print OUT param('criterion'), ln;
    print OUT param('mu'), ln;
    foreach $e ('rate') {
	for ($j=0; $j<$dim; $j++) {
	    print OUT param("$e:$j"), ' ';
	}
	print OUT ln;
    }

    foreach $e ('x') {
	for ($j=0; $j<4; $j++) {
	    print OUT param("$e:$j"), ' ';
	}
	print OUT ln;
    }
    close OUT;
    print '</pre>', ln;
}


if (param('Minimize')) {
    print_html_table();
    call_executable();
} elsif (param('Change')) {
    change_dim();
    print_html_table();
} else {
    print_html_table();
}

print '</BLOCKQUOTE>', end_html, ln;
